# -*- coding: utf-8 -*-

"""
Created on 2017-05-22
:author: Oshane Bailey (b4.oshany@gmail.com)
"""

from pytest import mark


def test_login_required(webtest, root):
    resp = webtest.get('/add_custom_content')
    assert resp.status_code == 302


@mark.user('admin')
def test_add(webtest, root):

    resp = webtest.get('/add_custom_content')

    # submit empty form
    form = resp.forms['deform']
    resp = form.submit('save')
    assert 'There was a problem' in resp.body

    # submit valid form
    form = resp.forms['deform']
    form['title'] = 'My Custom Content'
    form['custom_attribute'] = 'My Custom Attribute Value'
    resp = form.submit('save')
    assert resp.status_code == 302
    resp = resp.follow()
    assert 'Item was added.' in resp.body


@mark.user('admin')
def test_edit(webtest, root):

    from kotti_migration.resources import CustomContent

    root['cc'] = CustomContent(title=u'Content Title')

    resp = webtest.get('/cc/@@edit')
    form = resp.forms['deform']
    assert form['title'].value == u'Content Title'
    assert form['custom_attribute'].value == u''
    form['custom_attribute'] = u'Bazinga'
    resp = form.submit('save').maybe_follow()
    assert u'Your changes have been saved.' in resp.body
    assert u'Bazinga' in resp.body



# -*- coding: utf-8 -*-

"""
Created on 2017-05-22
:author: Oshane Bailey (b4.oshany@gmail.com)
"""

from kotti.interfaces import IDefaultWorkflow
from kotti.resources import Content
from sqlalchemy import Column
from sqlalchemy import ForeignKey
from sqlalchemy import Integer
from sqlalchemy import Unicode
from zope.interface import implements

from kotti_migration import _


class CustomContent(Content):
    """ A custom content type. """

    implements(IDefaultWorkflow)

    id = Column(Integer, ForeignKey('contents.id'), primary_key=True)
    custom_attribute = Column(Unicode(1000))

    type_info = Content.type_info.copy(
        name=u'CustomContent',
        title=_(u'CustomContent'),
        add_view=u'add_custom_content',
        addable_to=[u'Document'],
        selectable_default_views=[
            ("alternative-view", _(u"Alternative view")),
        ],
    )

    def __init__(self, custom_attribute=None, **kwargs):
        """ Constructor

        :param custom_attribute: A very custom attribute
        :type custom_attribute: unicode

        :param **kwargs: Arguments that are passed to the base class(es)
        :type **kwargs: see :class:`kotti.resources.Content`
        """

        super(CustomContent, self).__init__(**kwargs)

        self.custom_attribute = custom_attribute




import time
import uuid
import json
import re
import subprocess
import transaction
from uuid import UUID
from datetime import datetime, date
from unicodedata import normalize
from sqlalchemy.exc import IntegrityError

from kotti import DBSession
from kotti.resources import Content
from kotti.resources import LocalGroup
from kotti_migration import config


_punct_re = re.compile(r'[\t !"#$%&\'()*\-/<=>?@\[\\\]^_`{|},.:]+')


def uuid_factory():
    return uuid.uuid4()


def format_email_username(user):
    return '"{}" <{}>'.format(user.title, user.email)


def file_endswith(string, *args):
    extensions = '|'.join(args)
    return re.match(r'.*\.('+extensions+')$', string)


def chunk_list(ls, n):
    return [ls[i:i+n] for i in xrange(0, len(ls), n)]


def slugify(text, delim=u'-'):
    """Generates an slightly worse ASCII-only slug."""
    result = []
    text = unicode(text)
    for word in _punct_re.split(text.lower()):
        word = normalize('NFKD', word).encode('ascii', 'ignore')
        if word:
            result.append(word)
    return unicode(delim.join(result))


def unslugify(text, delim=u'-'):
    return text.replace(delim, " ")


def list_to_tuple(ls):
    return [(v,v) for v in ls]

def dict_to_tuple(dt):
    return [(k,v) for k,v in dt.items()]

def to_int(value, default=None, ceiling=True):
    try:
        if ceiling:
            return int(round(value))
        return int(value)
    except TypeError:
        return default


def json_value(value):
    if isinstance(value, datetime) or isinstance(value, date):
        return value.isoformat()
    if isinstance(value, list):
        return [json_value(v) for v in value]
    if isinstance(value, long):
        # Big numbers are sent as strings for accuracy in JavaScript
        if value > 9007199254740992 or value < -9007199254740992:
            return str(value)
    if isinstance(value, UUID):
        return str(value)
    if isinstance(value, LocalGroup):
        return {
            'principal_name': value.principal_name,
            'id': value.id,
            'node_id': value.node_id,
            'group_name': value.group_name
        }
    return value


def export(context, recursively=True):
    context_data = export_base(context)
    if recursively:
        context_data["children"] = export_children(context, recursively=recursively)
    return context_data


def export_children(context, recursively=True):
    children = context.children
    context_children = {}
    for child in children:
        context_children[child.name] = export(child, recursively=recursively)
    return context_children


def export_base(context):
    context_data = {}

    _data = context.__dict__

    keys = [k for k in context.__mapper__.columns.keys() if not k.startswith("_")]
    keys.append('local_groups')
    keys.append('position')

    for key in keys:
        context_data[key] = json_value(_data[key])

    context_data["__class__"] = "{0}.{1}".format(
        context.__class__.__module__,
        context.__class__.__name__
    )
    context_data["__foreign_keys__"] = {}
    for fk in context.__table__.foreign_keys:
        context_data["__foreign_keys__"][fk.parent.name] = "{}.{}".format(
            str(fk.column.table.name),
            str(fk.column.name)
        )
    return context_data


def import_data(base, context_data, recursively=True):
    _class_path = context_data["__class__"].rsplit(".", 1)
    if ".".join(_class_path) in config.ignore_content_types:
        return
    _class = get_content_type(
        content_type=_class_path[1],
        package=_class_path[0])

    default_attrs = {
        k: v for k,v in context_data.iteritems() if (
            not k.startswith("_") and k not in config.ignore_attrs
        )
    }
    base[context_data["name"]] = context = _class(**default_attrs)
    if recursively:
        import_children(base[context_data["name"]], context_data, recursively=recursively)



def import_children(context, context_data, recursively=True):
    children_data = context_data["children"]
    for child_name, child in children_data.iteritems():
        if child_name not in context:
            import_data(context, child)
        else:
            import_children(context[child_name], child)





def get_content_type(content_type, package="."):
    """ Get Content from kotti package.

    :param content_type:       Class name of the content type
    :param package:            Package path of the content type
    :returns:                  Content type

    """
    imported = getattr(__import__(package, fromlist=[content_type]), content_type)
    return imported


def find_key(d, val):
    return d.keys()[d.values().index(val)]


def remove_duplicates(list1):
    return reduce(lambda r, v: v in r and r or r + [v], list1, [])


def merge_list(list1, list2):
    """Remove duplicate values after merging the two lists"""
    return remove_duplicates((list1 or []) + (list2 or []))



# -*- coding: utf-8 -*-

"""
Created on 2017-05-22
:author: Oshane Bailey (b4.oshany@gmail.com)
"""

from kotti.resources import File
from pyramid.i18n import TranslationStringFactory
from kotti_migration import config as km_config

_ = TranslationStringFactory('kotti_migration')


def kotti_configure(settings):
    """ Add a line like this to you .ini file::

            kotti.configurators =
                kotti_migration.kotti_configure

        to enable the ``kotti_migration`` add-on.

    :param settings: Kotti configuration dictionary.
    :type settings: dict
    """

    settings['pyramid.includes'] += ' kotti_migration'
    settings['kotti.fanstatic.view_needed'] += ' kotti_migration.fanstatic.css_and_js'


def includeme(config):
    """ Don't add this to your ``pyramid_includes``, but add the
    ``kotti_configure`` above to your ``kotti.configurators`` instead.

    :param config: Pyramid configurator object.
    :type config: :class:`pyramid.config.Configurator`
    """
    config.add_static_view('static-kotti_migration', 'kotti_migration:static')

    m_ignores = config.registry.settings.get("migration.ignore_content_types", "")
    km_config.ignore_content_types = m_ignores.split("\n")


    config.scan(__name__)



import sys
import json
import re
import csv
import uuid
import time
import operator
import random
import transaction
import gzip
import click
import ConfigParser
import subprocess

from pyramid.paster import bootstrap


import sqlalchemy.orm.exc


def get_sql_url(config_file):
    """Returns the sql url from the configuration ini file"""
    config = ConfigParser.ConfigParser()
    config.readfp(open(config_file))
    return config.get('app:kotti', 'sqlalchemy.url', 1)


def get_login_url_db(sql_url=None, config_file=None):
    """ Returns the sql login url and database name from
        the configuration ini file or sql url.
    """
    if not sql_url:
        sql_url = get_sql_url(config_file)
    dbname_index = sql_url.rfind("/")
    dbname = sql_url[dbname_index+1:]
    login_url = sql_url[0:dbname_index]
    if '@' not in sql_url:
        login_url = sql_url
        dbname = sql_url.split('//')[1]
    if "?" in dbname:
        dbname = dbname[:dbname.index("?")]
    return login_url, dbname


def abort_if_false(ctx, param, value):
    if not value:
        ctx.abort()


@click.command()
@click.argument('config_file', type=click.Path('r'))
@click.argument('postgres_url')
@click.option('--login/--no-login', default=True,
              help="Use the SQL URL found in the configuration file to create the new site if --login is used, else use createdb and dropdb to carry out this task.")
@click.option('--yes', is_flag=True, callback=abort_if_false,
              help="Skip the prompt message and proceed with dropping and create a new site",
              expose_value=False,
              prompt='This will erase all pervious data, are you sure you want to create a new site?')
def clone_db(config_file, postgres_url, login, **kwargs):
    """ Drop the current database and clone the given database.

    Executing this command will drop and recreate the database for this application.

    Arguments:

        config_file:     Path to your init file from the current directory

        postgres_url:    URL of the database to clone into the existing database.

    """
    if not postgres_url.startswith("postgres://"):
        print "Invalid postgres url"
    start_time = time.time()
    sql_url = get_sql_url(config_file=config_file)
    login_url, dbname = get_login_url_db(sql_url=sql_url, config_file=config_file)
    print 'using the following database url: {}'.format(login_url)
    if login:
        subprocess.call(["psql", login_url, "-c",
                         "drop database if exists {};".format(dbname)])
        subprocess.call(["psql", login_url, "-c",
                         "create database {};".format(dbname)])
        subprocess.call(
            ["pg_dump {} | psql {}".format(postgres_url, sql_url)],
            shell=True
        )
    else:
        subprocess.call(["dropdb","--if-exists","{}".format(dbname)])
        subprocess.call(["createdb", "{}".format(dbname)])
        subprocess.call(
            ["pg_dump {} | psql {}".format(postgres_url, sql_url)],
            shell=True
        )
    print "{} to build a new site.".format(get_end_time(start_time))


def get_end_time(s_time):
    m, s = divmod(int(time.time() - s_time), 60)
    h, m = divmod(m, 60)
    return "It took %d hr(s) : %02d mins : %02d seconds" % (h, m, s)



import os

from setuptools import setup
from setuptools import find_packages

here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.rst')).read()
except IOError:
    README = ''
try:
    CHANGES = open(os.path.join(here, 'CHANGES.rst')).read()
except IOError:
    CHANGES = ''

version = "0.1.3"

install_requires = [
    'Kotti>=1.0.0',
    'kotti_tinymce',
    'click>=6.6',
]


setup(
    name='kotti_migration',
    version=version,
    description="Allow kotti content type to be imported and exported from a kotti site",
    long_description='\n\n'.join([README, CHANGES]),
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Framework :: Pylons",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        "License :: Repoze Public License",
    ],
    author='Oshane Bailey',
    author_email='b4.oshany@gmail.com',
    url='https://github.com/b4oshany/kotti_migration',
    keywords='kotti web cms wcms pylons pyramid sqlalchemy bootstrap migration export import',
    license="BSD-derived (http://www.repoze.org/LICENSE.txt)",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    tests_require=[],
    dependency_links=[],
    entry_points={
        'fanstatic.libraries': [
            'kotti_migration = kotti_migration.fanstatic:library',
        ],
        'console_scripts': [
            'kotti-clone-db = kotti_migration.scripts.migrator:clone_db',
        ]
    },
    extras_require={},
)



# -*- coding: utf-8 -*-

"""
Created on 2017-05-22
:author: Oshane Bailey (b4.oshany@gmail.com)
"""

import json
import colander
from kotti.views.edit import ContentSchema
from kotti.views.form import AddFormView
from kotti.views.form import EditFormView
from pyramid.view import view_config, view_defaults

from kotti_migration import _, util



@view_defaults(name="export",
               context="kotti.resources.Content",
               permission="admin")
class ExportImportView(object):

    def __init__(self, context, request):
        """ Constructor.
        :param context: Container of the nodes that will be created from
                        uploads.
        :type context: :class:`kotti.resources.Content` or descendants.
        :param request: Current request.
        :type request: :class:`kotti.request.Request`
        """

        self.context = context
        self.request = request

    @view_config(request_method="GET",
                 accept="application/json",
                 renderer="json")
    def export_data(self):
        context_data = util.export(self.context)
        return context_data


    @view_config(name="import",
                 renderer="kotti_migration:templates/import.pt")
    def import_data(self):
        if self.request.method == "POST":
            filename = self.request.POST['file'].filename
            input_file = self.request.POST['file'].file
            filepath = "/tmp"
            temp_file_path = filepath + '~'
            input_file.seek(0)
            data = json.load(input_file)
            util.import_data(base=self.context, context_data=data)
            self.request.session.flash(
                _(u"Data has been imported"),
                "success"
            )
        return {}

